(function() {

    const beginButton  = document.querySelector('#ngg-pro-braintree-begin');
    const cancelButton = document.querySelector('#ngg-pro-braintree-cancel');
    const submitButton = document.querySelector('#ngg-pro-braintree-submit');

    const modal   = document.querySelector('#braintree-dropin-parent-modal');
    const spinner = document.querySelector('#braintree-dropin-spinner');

    cancelButton.addEventListener('click', function(event) {
        event.preventDefault();
        modal.classList.remove('braintree-dropin-parent-modal-open');
        modal.classList.add('braintree-dropin-parent-modal-closed');
    });

    beginButton.addEventListener('click', function(event) {
        event.preventDefault();
        modal.classList.add('braintree-dropin-parent-modal-open');
        modal.classList.remove('braintree-dropin-parent-modal-closed');
    });

    const brainTreeConfiguration = {
        authorization: window.nextgen_pro_braintree_data.client_token,
        dataCollector: window.nextgen_pro_braintree_data.pass_client_data,
        selector: '#braintree-dropin-container'
    };

    if (window.nextgen_pro_braintree_data.enable_paypal) {
        brainTreeConfiguration.paypal = {
            amount: window.Ngg_Pro_Cart.get_instance().total,
            currency: window.nextgen_pro_braintree_data.currency,
            flow: 'checkout'
        }
    }

    async function submitOrder(url = "", data = {}) {
        const response = await fetch(
            url,
            {
                body: JSON.stringify(data),
                cache: 'no-cache',
                headers: { 'Content-Type': 'application/json' },
                method: 'POST'
            }
        );
        return response.json();
    }

    const brainTreeCallback = function (err, instance) {

        if (err) {
            console.log("Could not initialize Braintree; error provided:", err);
            beginButton.remove();
            return false;
        }

        submitButton.addEventListener('click', function(event) {
            event.preventDefault();

            // Prevent multiple submissions
            submitButton.setAttribute('disabled', 'disabled');
            submitButton.innerText = submitButton.dataset.processingMessage;
            spinner.classList.remove('braintree-dropin-hidden');

            instance.requestPaymentMethod(function (error, payload) {
                if (error) {
                    alert("An error occurred, please try again later. \nMessage: " + error);
                    submitButton.innerText = submitButton.dataset.normalMessage;
                    submitButton.removeAttribute('disabled');
                    spinner.classList.add('braintree-dropin-hidden');
                    return false;
                }

                submitOrder(
                    window.nextgen_pro_braintree_data.rest_url,
                    {
                        client_data: payload.deviceData,
                        nonce: payload.nonce,
                        cart: jQuery('#ngg_pro_checkout').serialize()
                    }
                ).then((data) => {
                    if (data.status === 'success') {
                        window.location = data.confirmation_url;
                    } else {
                        data.errors.forEach(error => alert('Error code: ' + error[0] + "\n" + "Error message: " + error[1]));
                    }
                });
            });
        });

        // In case the cart data updated after the initial page load
        instance.on('paymentOptionSelected', function() {
            instance.updateConfiguration('paypal', 'amount', window.Ngg_Pro_Cart.get_instance().total);
            instance.updateConfiguration('paypal', 'currency', window.nextgen_pro_braintree_data.currency);
        });

        // Payment methods with external flow (PayPal, Venmo, etc) fire this event as soon as their process has completed.
        // Form based payment methods (credit cards) fires when all fields pass client side validation, even though the
        // customer may not yet be ready to submit their data. Here event.paymentMethodIsSelected is true when the payment
        // method is presented as selected and false when it is not.
        instance.on('paymentMethodRequestable', function(event) {
            // Automatic capture is possible if event.paymentMethodIsSelected is true, but to avoid confusion we will
            // require customers to click one more time to finalize the submission.
            submitButton.classList.remove('braintree-dropin-hidden');
            submitButton.removeAttribute('disabled');
        });

        instance.on('noPaymentMethodRequestable', function(event) {
            submitButton.classList.add('braintree-dropin-hidden');
            submitButton.setAttribute('disabled', 'disabled');
        });
    };

    braintree.dropin.create(brainTreeConfiguration, brainTreeCallback);

})();
